package com.dgtis.data.api;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.dgtis.common.core.utils.DateUtils;
import com.dgtis.common.core.utils.StringUtils;
import com.dgtis.common.core.utils.poi.CsvUtil;
import com.dgtis.common.core.utils.poi.ExcelUtil;
import com.dgtis.common.core.web.domain.AjaxResult;
import com.dgtis.common.security.utils.SecurityUtils;
import com.dgtis.data.domain.Customer;
import com.dgtis.data.service.ISysUserService;
import com.dgtis.data.util.AddressResolutionUtil;
import com.dgtis.system.api.domain.SysUser;
import com.mysql.cj.x.protobuf.MysqlxDatatypes;
import org.apache.http.HttpEntity;
import org.apache.http.entity.ContentType;
import org.apache.http.nio.entity.NStringEntity;
import org.apache.http.util.EntityUtils;
import org.elasticsearch.client.Request;
import org.elasticsearch.client.Response;
import org.elasticsearch.client.RestClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.text.DecimalFormat;
import java.util.*;

/**
 * @description:客户信息列表api
 * @author:qxm
 * @date:2021/1/22 10:24
 */
@Controller
@RequestMapping("/customer")
public class CustomerApiController {

    private static Logger logger = LoggerFactory.getLogger(CustomerApiController.class);
    @Autowired
    private RestClient restClient;
    @Autowired
    private ISysUserService userService;

    @GetMapping("/getCustomerList")
    @ResponseBody
    public AjaxResult getCustomerList(@RequestParam(defaultValue = "1") int pageNum,@RequestParam(defaultValue = "10") int pageSize,
          String education,//学历
          String policybelong,//业绩归属
          String payS,String payE,  //年缴费区间
          String fadate,//投保年份
          String productname,//险种类别
          String insuredAmountS,String  insuredAmountE,//保额区间
          String ageS,String ageE,//年龄区间
          String gender,//性别
          String sobirth,//省份
          String NOVPolicy,//保单件数
          String custtype,//客户类型  投保人
          String SCustID,// 核心客户号
          String custclass,//客户等级
          String participantsDateS,String participantsDateE,//参与方时间
          String PIncomeS,String PIncomeE,//年收入区间
          String Insured,//投保对象
          String lp,//理赔
          String yx,//有效
          String gzgw,//关注官微
          String zcgw,//注册官微
          String name,//客户姓名
          String idcard,//客户证件号码
          String mobile//客户手机号
        ) {

        try {
            logger.info("进客户列表时间："+DateUtils.getTime());
            /*Long userId = SecurityUtils.getUserId();
            SysUser sysUser = userService.selectUserById(userId);
            String deptCode = sysUser.getDept().getDeptCode();*/

            //2.如果没有客户信息以外的查询条件，只需要查询客户信息

            // 获取索引的别名，字段，创建时间
            String beginBirthDay = "";
            String endBirthDay = "";
            if(StringUtils.isNotEmpty(ageS)){
                String year = DateUtils.getYYYY();
                int bYear = Integer.parseInt(year)-Integer.parseInt(ageS);
                endBirthDay = bYear+"-"+DateUtils.getMMDD();
            }

            if(StringUtils.isNotEmpty(ageE)){
                String year = DateUtils.getYYYY();
                int bYear = Integer.parseInt(year)-Integer.parseInt(ageE);
                beginBirthDay = bYear+"-"+DateUtils.getMMDD();
            }

            //拼装查询条件
            StringBuilder builder =new StringBuilder();
            builder.append("{");
            builder.append("\"query\":{");//query开始
            builder.append("\"bool\":{");//boot开始
            builder.append("\"must\":[");//must开始
            //业绩归属字段
            /*if(StringUtils.isNotEmpty(deptCode)){
                builder.append("{\"prefix\":{\"policybelong\":\""+deptCode+"\"}},");
            }*/

            if(StringUtils.isNotEmpty(policybelong)){
                builder.append("{\"prefix\":{\"policybelong\":\""+policybelong+"\"}},");
            }

            //客户姓名
            if(StringUtils.isNotEmpty(name)){
                builder.append("{\"wildcard\":{\"name\":\"*"+name+"*\"}},");
            }

            //客户手机
            if(StringUtils.isNotEmpty(mobile)){
                builder.append("{\"wildcard\":{\"pmphone\":\"*"+mobile+"*\"}},");
            }

            //投保年份
            if(StringUtils.isNotEmpty(fadate)){
                builder.append("{\"wildcard\":{\"fadate\":\"*"+fadate+"*\"}},");
            }

            if(StringUtils.isNotEmpty(gender)){
                builder.append("{\"prefix\":{\"gender\":"+gender+"}},");
            }
            if(StringUtils.isNotEmpty(NOVPolicy)){
                builder.append("{\"prefix\":{\"label25\":\""+NOVPolicy+"\"}},");
            }
            if(StringUtils.isNotEmpty(custclass)){
                builder.append("{\"term\":{\"custclass\":\""+custclass+"\"}},");
            }
            if(StringUtils.isNotEmpty(sobirth)){
                builder.append("{\"wildcard\":{\"sobirth\":\"*"+sobirth+"*\"}},");//模糊匹配
            }

            if(StringUtils.isNotEmpty(SCustID)){
                builder.append("{\"wildcard\":{\"scustid\":\"*"+SCustID+"*\"}},");//模糊匹配
            }
            if(StringUtils.isNotEmpty(custtype)){
                builder.append("{\"wildcard\":{\"custtype\":\"*"+custtype+"*\"}},");
            }

            if(StringUtils.isNotEmpty(beginBirthDay) && StringUtils.isNotEmpty(endBirthDay)){
                builder.append("{\"range\":{\"birthday\":{\"gte\":\""+beginBirthDay+"\",\"lte\":\""+endBirthDay+"\"}}},");
            }else{

                if(StringUtils.isNotEmpty(beginBirthDay)){
                    builder.append("{\"range\":{\"birthday\":{\"gte\":\""+beginBirthDay+"\"}}},");
                }
                if(StringUtils.isNotEmpty(endBirthDay)){
                    builder.append("{\"range\":{\"birthday\":{\"lte\":\""+endBirthDay+"\"}}},");
                }
            }


            if(StringUtils.isNotEmpty(participantsDateS) && StringUtils.isNotEmpty(participantsDateE)){
                builder.append("{\"range\":{\"fadate\":{\"gte\":\""+participantsDateS+"\",\"lte\":\""+participantsDateE+"\"}}},");
            }else{

                if(StringUtils.isNotEmpty(participantsDateS)){
                    builder.append("{\"range\":{\"fadate\":{\"gte\":\""+participantsDateS+"\"}}},");
                }
                if(StringUtils.isNotEmpty(participantsDateE)){
                    builder.append("{\"range\":{\"fadate\":{\"lte\":\""+participantsDateE+"\"}}},");
                }
            }

            if(StringUtils.isNotEmpty(PIncomeS) && StringUtils.isNotEmpty(PIncomeE)){
                builder.append("{\"range\":{\"pincome\":{\"gte\":\""+PIncomeS+"\",\"lte\":\""+PIncomeE+"\"}}},");
            }else{

                if(StringUtils.isNotEmpty(PIncomeS)){
                    builder.append("{\"range\":{\"pincome\":{\"gte\":\""+PIncomeS+"\"}}},");
                }
                if(StringUtils.isNotEmpty(PIncomeE)){
                    builder.append("{\"range\":{\"pincome\":{\"lte\":\""+PIncomeE+"\"}}},");
                }
            }

            if(StringUtils.isNotEmpty(yx)){
                if (yx.equals("0")){
                    builder.append("{\"range\":{\"novpolicy\":{\"gt\":\""+yx+"\"}}},");
                }else
                {
                    builder.append("{\"prefix\":{\"label91\":\"失效客户\"}},");
                }
            }

            if(StringUtils.isNotEmpty(education)){
                String[] edus = education.split("&");
                builder.append("{\"terms\":{\"education\":[");
                for (String edu:edus) {
                    builder.append("\""+edu+"\",");
                }
                builder.delete(builder.length()-1,builder.length());
                builder.append ("]}},");
            }

            if(StringUtils.isNotEmpty(gzgw) && "0".equals(gzgw)){
                builder.append("{\"prefix\":{\"label73\":\"关注官微\"}},");
            }

            if(StringUtils.isNotEmpty(zcgw) && "0".equals(zcgw)){
                builder.append("{\"prefix\":{\"label74\":\"注册官微\"}},");
            }

            if(StringUtils.isNotEmpty(lp) && "0".equals(lp)){
                builder.append("{\"prefix\":{\"label57\":\"最近一次理赔类型\"}},");
            }

            if(StringUtils.isNotEmpty(gender) || StringUtils.isNotEmpty(NOVPolicy) || StringUtils.isNotEmpty(custtype)
                    || StringUtils.isNotEmpty(custclass) || StringUtils.isNotEmpty(sobirth) || StringUtils.isNotEmpty(SCustID)
                    || StringUtils.isNotEmpty(beginBirthDay) || StringUtils.isNotEmpty(endBirthDay) || StringUtils.isNotEmpty(education)
                    || StringUtils.isNotEmpty(PIncomeS) || StringUtils.isNotEmpty(PIncomeE) || (StringUtils.isNotEmpty(gzgw) && "0".equals(gzgw))
                    || (StringUtils.isNotEmpty(zcgw) && "0".equals(zcgw)) || StringUtils.isNotEmpty(yx)
                    || (StringUtils.isNotEmpty(lp) && "0".equals(lp)) || /*StringUtils.isNotEmpty(deptCode) ||*/
                    StringUtils.isNotEmpty(participantsDateS) || StringUtils.isNotEmpty(participantsDateE) ||
                    StringUtils.isNotEmpty(policybelong) || StringUtils.isNotEmpty(name) || StringUtils.isNotEmpty(mobile)
                    || StringUtils.isNotEmpty(fadate)){
                builder.delete(builder.length()-1,builder.length());
            }

            builder.append("],");//must结束
            builder.append("\"must_not\":[");//must_not
            if(StringUtils.isNotEmpty(gzgw) && "1".equals(gzgw)){
                builder.append("{\"prefix\":{\"label73\":\"关注官微\"}},");
            }

            if(StringUtils.isNotEmpty(zcgw) && "1".equals(zcgw)){
                builder.append("{\"prefix\":{\"label74\":\"注册官微\"}},");
            }

            if(StringUtils.isNotEmpty(lp) && "1".equals(lp)){
                builder.append("{\"prefix\":{\"label57\":\"最近一次理赔类型\"}},");
            }

            if(
                    (StringUtils.isNotEmpty(gzgw) && "1".equals(gzgw))
                    || (StringUtils.isNotEmpty(zcgw) && "1".equals(zcgw))
                    || (StringUtils.isNotEmpty(lp) && "1".equals(lp))
                ){
                builder.delete(builder.length()-1,builder.length());
            }

            builder.append("],");
            builder.append("\"should\":[");

            //客户证件号码
            if(StringUtils.isNotEmpty(idcard)){
                builder.append("{\"wildcard\":{\"idcard\":\"*"+idcard+"*\"}},");
                builder.append("{\"wildcard\":{\"passport\":\"*"+idcard+"*\"}},");
                builder.append("{\"wildcard\":{\"dlicense\":\"*"+idcard+"*\"}},");
                builder.append("{\"wildcard\":{\"otheridnumber\":\"*"+idcard+"*\"}},");
                builder.delete(builder.length()-1,builder.length());
            }


            builder.append("]");//should
            builder.append("}");//boot结束
            builder.append("},");//query结束
            builder.append("\"from\":"+((pageNum-1)*10)+",");//第几页
            builder.append("\"size\":"+pageSize+",");//查询几个
            builder.append("\"sort\":[{\"scustid\":{\"order\":\"asc\"}}],");//排序
            builder.append("\"aggs\":{}");//
            builder.append("}");


            HttpEntity entity = new NStringEntity(builder.toString(), ContentType.APPLICATION_JSON);
            String url = "/shanglifeecif.individual/_search?pretty=true";
            Request scriptRequest = new Request("GET", url);
            scriptRequest.setEntity(entity);
            Response response = restClient.performRequest(scriptRequest);
            String result = EntityUtils.toString(response.getEntity());
            JSONObject jsonObject = JSON.parseObject(result);
            JSONObject hitsobject = (JSONObject) jsonObject.get("hits");
            JSONArray array = JSON.parseArray(hitsobject.get("hits").toString());
            List listMap = new ArrayList<>();
            Map map = new HashMap();
            for (int i = 0; i <array.size() ; i++) {
                JSONObject json = (JSONObject)array.get(i);
                Map mp = new HashMap();
                mp.put("id",json.getString("_id"));
                JSONObject jsonOb = JSON.parseObject(json.getString("_source"));
                mp.putAll(JSON.parseObject(json.getString("_source"),Map.class));

                if(mp.containsKey("birthday")){
                    mp.put("birthday", jsonOb.getString("birthday"));
                    mp.put("age",DateUtils.getAge(DateUtils.dateTime(DateUtils.YYYY_MM_DD,jsonOb.getString("birthday"))));
                }else{
                    mp.put("birthday", "");
                    mp.put("age","");
                }
                listMap.add(mp);
            }
            long total = hitsobject.getLong("total");

            map.put("list",listMap);
            map.put("total",total);
            logger.info("出客户列表时间："+DateUtils.getTime());
            return AjaxResult.success(map);
        } catch (Exception e) {
            logger.info("列表报错时间："+DateUtils.getTime());
            e.printStackTrace();
            return AjaxResult.error();
        }
    }

    @GetMapping("/getCustomerById")
    @ResponseBody
    public AjaxResult getCustomerById(String id) {
        // 获取索引的别名，字段，创建时间http://10.32.2.231:9200/shanglifeecif.individual/default_type_/1
      try {
           logger.info("进客户详情时间："+DateUtils.getTime());
            Map<String, String> paramMap = new HashMap<String, String>();
            paramMap.put("pretty", "true");
            Request scriptRequest = new Request("GET", "/shanglifeecif.individual/default_type_/"+id+"?pretty=true");

            Response response = restClient.performRequest(scriptRequest);
            String result = EntityUtils.toString(response.getEntity());
            Map map = new HashMap();
            JSONObject jsonObject = JSON.parseObject(result);
            Map mpInfo = new HashMap();
          String indid = jsonObject.getString("_id");

          mpInfo.put("id",indid);
          JSONObject source = JSON.parseObject(jsonObject.getString("_source"));
          String scustid = source.getString("scustid");
          String custtype = source.getString("custtype");
          mpInfo.putAll(JSON.parseObject(jsonObject.getString("_source"),Map.class));
            if(mpInfo.containsKey("birthday")){
                mpInfo.put("birthday", source.getString("birthday"));
                mpInfo.put("age",DateUtils.getAge(DateUtils.dateTime(DateUtils.YYYY_MM_DD,source.getString("birthday"))));

            }else{
                mpInfo.put("birthday","");
                mpInfo.put("age","");
            }
            map.put("custInfo",mpInfo);
          Map query  =new HashMap();
          query.put("indid1",indid);
           scriptRequest = new Request("GET", "/shanglifeecif.indrelationship/_search?pretty=true&q=indid1:"+scustid);
          response = restClient.performRequest(scriptRequest);
          result = EntityUtils.toString(response.getEntity());
          jsonObject = JSON.parseObject(result);
          JSONObject hitsobject = (JSONObject) jsonObject.get("hits");
          JSONArray array = JSON.parseArray(hitsobject.get("hits").toString());
          List relationship = new ArrayList<>();
          for (int i = 0; i <array.size() ; i++) {
              JSONObject json = (JSONObject)array.get(i);
              Map mp = new HashMap();
              mp.put("id",json.getString("_id"));
              mp.putAll(JSON.parseObject(json.getString("_source"),Map.class));
              relationship.add(mp);
          }
            map.put("relationship",relationship);
          query  =new HashMap();
          query.put("applicantid",indid);
          scriptRequest = new Request("GET", "/shanglifeecif.insuranceclaimthread/_search?size=100&pretty=true&q=lpscutid:"+scustid);

          response = restClient.performRequest(scriptRequest);
          result = EntityUtils.toString(response.getEntity());
          jsonObject = JSON.parseObject(result);
          hitsobject = (JSONObject) jsonObject.get("hits");
          array = JSON.parseArray(hitsobject.get("hits").toString());
          List insuranceclaimthread = new ArrayList<>();
          for (int i = 0; i <array.size() ; i++) {
              JSONObject json = (JSONObject)array.get(i);
              Map mp = new HashMap();
              mp.put("id",json.getString("_id"));
              mp.putAll(JSON.parseObject(json.getString("_source"),Map.class));
              insuranceclaimthread.add(mp);
          }
          map.put("insuranceclaimthread",insuranceclaimthread);

          if("被保人".equals(custtype.trim())){
              scriptRequest = new Request("GET", "/shanglifeecif.insurancearrangement/_search?size=100&pretty=true&sort=padate:asc&q=insuredscustid:"+scustid);
          }else{
              scriptRequest = new Request("GET", "/shanglifeecif.insurancearrangement/_search?size=100&pretty=true&sort=padate:asc&q=applicantscustid:"+scustid);
          }

          response = restClient.performRequest(scriptRequest);
          result = EntityUtils.toString(response.getEntity());
          jsonObject = JSON.parseObject(result);
          hitsobject = (JSONObject) jsonObject.get("hits");
          array = JSON.parseArray(hitsobject.get("hits").toString());
          List insurancearrangement = new ArrayList<>();
          for (int i = 0; i <array.size() ; i++) {
              JSONObject json = (JSONObject)array.get(i);
              Map mp = new HashMap();
              mp.put("id",json.getString("_id"));
              mp.putAll(JSON.parseObject(json.getString("_source"),Map.class));
              insurancearrangement.add(mp);
          }
           map.put("insurancearrangement",insurancearrangement);
           scriptRequest = new Request("GET", "/shanglifeecif.partytimeline/_search?size=100&pretty=true&sort=sdate:asc&q=partyid:"+scustid);

           response = restClient.performRequest(scriptRequest);
           result = EntityUtils.toString(response.getEntity());
           jsonObject = JSON.parseObject(result);
           hitsobject = (JSONObject) jsonObject.get("hits");
           array = JSON.parseArray(hitsobject.get("hits").toString());
           List partytimeline = new ArrayList<>();
           for (int i = 0; i <array.size() ; i++) {
              JSONObject json = (JSONObject)array.get(i);
              Map mp = new HashMap();
              mp.put("id",json.getString("_id"));
              mp.putAll(JSON.parseObject(json.getString("_source"),Map.class));
              partytimeline.add(mp);
           }
           map.put("partytimeline",partytimeline);
            logger.info("出客户详情时间："+DateUtils.getTime());
            return AjaxResult.success(map);
        } catch (Exception e) {
            logger.info("客户详情报错时间："+DateUtils.getTime());
            e.printStackTrace();
            return AjaxResult.error();
        }
    }

    @GetMapping("/getCustomerOverViewById")
    @ResponseBody
    public AjaxResult getCustomerOverViewById(String id) {
        // 获取索引的别名，字段，创建时间http://10.32.2.231:9200/shanglifeecif.individual/default_type_/1
        try {
            logger.info("进客户概览时间："+DateUtils.getTime());

            Request scriptRequest = new Request("GET", "shanglifeecif.individual/default_type_/"+id+"?pretty=true");
            Response response = restClient.performRequest(scriptRequest);

            String result = EntityUtils.toString(response.getEntity());
            Map map = new HashMap();
            JSONObject jsonObject = JSON.parseObject(result);
            Map mpInfo = new HashMap();
            String indid = jsonObject.getString("_id");
            mpInfo.put("id",indid);
            JSONObject source = JSON.parseObject(jsonObject.getString("_source"));
            String scustid = source.getString("scustid");
            mpInfo.putAll(JSON.parseObject(jsonObject.getString("_source"),Map.class));
            if(mpInfo.containsKey("birthday")){
                mpInfo.put("birthday", source.getString("birthday"));
                mpInfo.put("age",DateUtils.getAge(DateUtils.dateTime(DateUtils.YYYY_MM_DD,source.getString("birthday"))));
            }else{
                mpInfo.put("birthday","");
                mpInfo.put("age","");
            }
           scriptRequest = new Request("GET", "/shanglifeecif.insurancearrangement/_search?size=100&pretty=true&sort=padate:asc&q=applicantscustid:"+scustid);
            response = restClient.performRequest(scriptRequest);
            result = EntityUtils.toString(response.getEntity());
            jsonObject = JSON.parseObject(result);
            JSONObject hitsobject = (JSONObject) jsonObject.get("hits");
            JSONArray array = JSON.parseArray(hitsobject.get("hits").toString());
            DecimalFormat   df  = new DecimalFormat("######0.00");//保留两位小数
            double prem = 0;//标准保费
            double sumprem = 0;//累计保费
            double handedinprem = 0;//已交保费
            //概览投保时间和保费折线图
            Map expectedData = new HashMap();
            String[] edata = new String[]{"0","0","0","0","0","0","0","0","0","0","0","0"};
            expectedData.put("name","");

            Map actaulData = new HashMap();
            String[] adata = new String[]{"0","0","0","0","0","0","0","0","0","0","0","0"};
            actaulData.put("name","");
            List xaxisData = new ArrayList();
            xaxisData.add("1");
            xaxisData.add("2");
            xaxisData.add("3");
            xaxisData.add("4");
            xaxisData.add("5");
            xaxisData.add("6");
            xaxisData.add("7");
            xaxisData.add("8");
            xaxisData.add("9");
            xaxisData.add("10");
            xaxisData.add("11");
            xaxisData.add("12");
            long insureDay = 0;//投保时长
            int guaranteeDay = 0;//享受保障天数
            Map majorCategorie = new HashMap();
            Map mpremMap = new HashMap();
            int firstDate = 1;//定义第一次长期险获取
            Date date = new Date();
            String currentDateTime = DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD,date);
            String currentYear = DateUtils.parseDateToStr(DateUtils.YYYY,date);
            //获取最近年份
            for (int i = 0; i <array.size() ; i++) {
                JSONObject json = (JSONObject)array.get(i);
                JSONObject _source = (JSONObject) json.get("_source");
                Date padate = _source.getDate("padate");

                if(i==0){
                    insureDay = DateUtils.getDateDifDay(new Date(),padate);
                }
                if("有效".equals(_source.getString("policystate"))){

                    if(_source.containsKey("sumprem")){
                        sumprem+=_source.getDouble("sumprem");

                    }

                    Integer agrmntage = _source.getInteger("agrmntage");
                    //短期险
                    if(agrmntage<180){
                        Date pindate = _source.getDate("pindate");//起保日期
                        Date pmdate = _source.getDate("pmdate");//终保日期
                        guaranteeDay+=DateUtils.getDateDifDay(pmdate,pindate);
                    }else{
                        //获取第一次长期险的投保日期，使用当前日期减去投保日期获取保障天数
                        if(firstDate==1){
                            guaranteeDay+=DateUtils.getDateDifDay(new Date(),padate);
                            firstDate++;
                        }
                    }


                    if(_source.containsKey("prem")){
                        //计算年缴保费过滤掉缴费方式趸缴的
                        if(!"趸缴".equals(_source.getString("payment"))){
                            prem+=_source.getDouble("prem");
                            //String currentPadateStr = DateUtils.parseDateToStr(currentYear+"-MM-dd",padate);
                            if(_source.containsKey("npdate")){
                                Date npdate = _source.getDate("npdate");
                                String npDateStr = DateUtils.parseDateToStr("yyyy-MM-dd", npdate);
                                String npDateStrYear = DateUtils.parseDateToStr("yyyy", npdate);
                                if(npDateStr.compareTo(currentDateTime)>0 && !npDateStrYear.equals(currentYear)){
                                    handedinprem+=_source.getDouble("prem");
                                }
                            }

                            String m = DateUtils.parseDateToStr("M", padate);

                            if(mpremMap.containsKey(m)){
                                double tprem = Double.parseDouble(mpremMap.get(m).toString());
                                tprem+=_source.getDouble("prem");
                                mpremMap.put(m,df.format(tprem));
                                edata[Integer.parseInt(m)-1] = df.format(tprem);
                                adata[Integer.parseInt(m)-1] = df.format(tprem);
                            }else{
                                mpremMap.put(m,df.format(_source.getDouble("prem")));
                                edata[Integer.parseInt(m)-1] = df.format(_source.getDouble("prem"));
                                adata[Integer.parseInt(m)-1] = df.format(_source.getDouble("prem"));
                            }
                        }else{
                            if("趸缴".equals(_source.getString("payment")) && currentYear.equals(DateUtils.parseDateToStr("yyyy",padate))){
                                prem+=_source.getDouble("prem");
                            }


                        }

                    }

                    String risk_categories_name = _source.getString("risk_categories_name");

                    if(majorCategorie.containsKey(risk_categories_name)){
                        double risk =  Double.parseDouble(majorCategorie.get(risk_categories_name).toString());
                        if(_source.containsKey("risk")){
                            risk += _source.getDouble("risk");
                        }
                        majorCategorie.put(risk_categories_name,risk);
                    }else{
                        if(_source.containsKey("risk") && StringUtils.isNotEmpty(risk_categories_name)){
                            majorCategorie.put(risk_categories_name,_source.getDouble("risk"));
                        }
                    }

                }else{
                    //客户的所有保单缴费期满且承保期限已满正常终止的客户,需要加上累计保费
                    if("终止".equals(_source.getString("policystate"))){
                        String norenewal = _source.getString("norenewal")==null?"":_source.getString("norenewal").trim();//续保次数
                        String payendyear = _source.getString("payendyear")==null?"":_source.getString("payendyear").trim();//缴费年期
                        Date pmdate = _source.getDate("pmdate");//终保日期
                        if(norenewal.equals(payendyear) && currentDateTime.compareTo(DateUtils.parseDateToStr(DateUtils.YYYY_MM_DD,pmdate))>0){
                            if(_source.containsKey("sumprem")){
                                sumprem+=_source.getDouble("sumprem");

                            }
                        }
                    }
                }

            }

            expectedData.put("data",edata);
            actaulData.put("data",adata);
            Map lineBarChartData = new HashMap();
            lineBarChartData.put("expectedData",expectedData);
            lineBarChartData.put("actaulData",actaulData);
            lineBarChartData.put("xaxisData",xaxisData);
            map.put("lineBarChartData",lineBarChartData);
            //查询理赔信息
            scriptRequest = new Request("GET", "/shanglifeecif.insuranceclaimthread/_search?size=100&pretty=true&q=lpscutid:"+scustid);
            response = restClient.performRequest(scriptRequest);
            result = EntityUtils.toString(response.getEntity());
            jsonObject = JSON.parseObject(result);
            hitsobject = (JSONObject) jsonObject.get("hits");
            array = JSON.parseArray(hitsobject.get("hits").toString());
            Map insuranceclaimthread = new HashMap();
            List insuranceclaimthreads = new ArrayList<>();
            double claimAmount = 0;//理赔总金额
            Set<String> claimTypeSet = new HashSet<>();//理赔类型
            for (int i = 0; i <array.size() ; i++) {
                JSONObject json = (JSONObject)array.get(i);
                JSONObject claimSource = JSON.parseObject(json.getString("_source"));

                if(claimSource.containsKey("ccamt")){
                    claimAmount+=claimSource.getDouble("ccamt");
                }
                claimTypeSet.add(claimSource.getString("accidenttype"));
                insuranceclaimthreads.add(claimSource);
            }
            insuranceclaimthread.put("isClaim",insuranceclaimthreads.size()>0?"是":"否");
            insuranceclaimthread.put("claimAmount",df.format(claimAmount));
            insuranceclaimthread.put("claimType",String.join(",",claimTypeSet));
            map.put("insuranceclaimthread",insuranceclaimthread);
            mpInfo.put("prem",df.format(prem));//年缴保费/应交保费
            mpInfo.put("sumprem",df.format(sumprem));//累计保费
            mpInfo.put("handedinprem",df.format(handedinprem));//已交保费
            mpInfo.put("insureDay",(insureDay/365) +"年"+(insureDay%365)+"天");
            //保障天数需要加上退保之前的天数
            scriptRequest = new Request("GET", "/shanglifeecif.surrenderprotectiontemp/_search?pretty=true&q=scustid:"+scustid);
            response = restClient.performRequest(scriptRequest);
            result = EntityUtils.toString(response.getEntity());
            jsonObject = JSON.parseObject(result);
            hitsobject = (JSONObject) jsonObject.get("hits");
            array = JSON.parseArray(hitsobject.get("hits").toString());
            for (int i = 0; i <array.size() ; i++) {
                JSONObject json = (JSONObject)array.get(i);
                JSONObject tbSource = JSON.parseObject(json.getString("_source"));
                if(tbSource.containsKey("tbday")){
                    guaranteeDay+=tbSource.getInteger("tbday");
                }
            }

            mpInfo.put("guaranteeDay",guaranteeDay>insureDay?insureDay+"天":guaranteeDay+"天");
            map.put("custInfo",mpInfo);
            map.put("majorCategorie",majorCategorie);
            logger.info("出客户概览时间："+DateUtils.getTime());
            return AjaxResult.success(map);
        } catch (Exception e) {
            logger.info("客户概览报错时间："+DateUtils.getTime());
            e.printStackTrace();
            return AjaxResult.error();
        }
    }

    @GetMapping("/export")
    public void export(HttpServletResponse response, HttpServletRequest request,
           String education,//学历
           String policybelong,//业绩归属
           String payS,String payE,  //年缴费区间
           String fadate,//投保年份
           String productname,//险种类别
           String insuredAmountS,String  insuredAmountE,//保额区间
           String ageS,String ageE,//年龄区间
           String gender,//性别
           String sobirth,//省份
           String NOVPolicy,//保单件数
           String custtype,//客户类型  投保人
           String SCustID,// 核心客户号
           String custclass,//客户等级
           String participantsDateS,String participantsDateE,//参与方时间
           String PIncomeS,String PIncomeE,//年收入区间
           String Insured,//投保对象
           String lp,//理赔
           String yx,//有效
           String gzgw,//关注官微
           String zcgw,//注册官微
           String name,//客户姓名
           String idcard,//客户证件号码
           String mobile//客户手机号
    ) {
        logger.info("导出进时间："+DateUtils.getTime());
        try{
           /* Long userId = SecurityUtils.getUserId();
            SysUser sysUser = userService.selectUserById(userId);
            String deptCode = sysUser.getDept().getDeptCode();*/
            //1.如果存在保单信息的查询条件，需要先查询保单信息然后通过保单关联出客户信息
            //处理保单查询条件
            /*List<String> customerIds = new ArrayList<String>();
            if(StringUtils.isNotEmpty(payS) || StringUtils.isNotEmpty(payE)
                    || StringUtils.isNotEmpty(insuredAmountS) || StringUtils.isNotEmpty(insuredAmountE)
                    || StringUtils.isNotEmpty(productname) || StringUtils.isNotEmpty(fadate)){
                customerIds = queryInsurancearrangement(policybelong,payS,payE,fadate,productname,insuredAmountS,insuredAmountE);
            }*/


            //2.如果没有客户信息以外的查询条件，只需要查询客户信息

            // 获取索引的别名，字段，创建时间
            String beginBirthDay = "";
            String endBirthDay = "";
            if(StringUtils.isNotEmpty(ageS)){
                String year = DateUtils.getYYYY();
                int bYear = Integer.parseInt(year)-Integer.parseInt(ageS);
                endBirthDay = bYear+"-"+DateUtils.getMMDD();
            }

            if(StringUtils.isNotEmpty(ageE)){
                String year = DateUtils.getYYYY();
                int bYear = Integer.parseInt(year)-Integer.parseInt(ageE);
                beginBirthDay = bYear+"-"+DateUtils.getMMDD();
            }

            //拼装查询条件
            StringBuilder builder =new StringBuilder();
            builder.append("{");
            builder.append("\"query\":{");//query开始
            builder.append("\"bool\":{");//boot开始
            builder.append("\"must\":[");//must开始
            //业绩归属字段
            /*if(StringUtils.isNotEmpty(deptCode)){
                builder.append("{\"prefix\":{\"policybelong\":\""+deptCode+"\"}},");
            }*/

            if(StringUtils.isNotEmpty(policybelong)){
                builder.append("{\"prefix\":{\"policybelong\":\""+policybelong+"\"}},");
            }

            //客户姓名
            if(StringUtils.isNotEmpty(name)){
                builder.append("{\"wildcard\":{\"name\":\"*"+name+"*\"}},");
            }

            //客户手机
            if(StringUtils.isNotEmpty(mobile)){
                builder.append("{\"wildcard\":{\"pmphone\":\"*"+mobile+"*\"}},");
            }

            //投保年份
            if(StringUtils.isNotEmpty(fadate)){
                builder.append("{\"wildcard\":{\"fadate\":\"*"+fadate+"*\"}},");
            }

            if(StringUtils.isNotEmpty(gender)){
                builder.append("{\"prefix\":{\"gender\":"+gender+"}},");
            }
            if(StringUtils.isNotEmpty(NOVPolicy)){
                builder.append("{\"prefix\":{\"label25\":\""+NOVPolicy+"\"}},");
            }
            if(StringUtils.isNotEmpty(custclass)){
                builder.append("{\"term\":{\"custclass\":\""+custclass+"\"}},");
            }
            if(StringUtils.isNotEmpty(sobirth)){
                builder.append("{\"wildcard\":{\"sobirth\":\"*"+sobirth+"*\"}},");//模糊匹配
            }

            if(StringUtils.isNotEmpty(SCustID)){
                builder.append("{\"wildcard\":{\"scustid\":\"*"+SCustID+"*\"}},");//模糊匹配
            }
            if(StringUtils.isNotEmpty(custtype)){
                builder.append("{\"wildcard\":{\"custtype\":\"*"+custtype+"*\"}},");
            }

            if(StringUtils.isNotEmpty(beginBirthDay) && StringUtils.isNotEmpty(endBirthDay)){
                builder.append("{\"range\":{\"birthday\":{\"gte\":\""+beginBirthDay+"\",\"lte\":\""+endBirthDay+"\"}}},");
            }else{

                if(StringUtils.isNotEmpty(beginBirthDay)){
                    builder.append("{\"range\":{\"birthday\":{\"gte\":\""+beginBirthDay+"\"}}},");
                }
                if(StringUtils.isNotEmpty(endBirthDay)){
                    builder.append("{\"range\":{\"birthday\":{\"lte\":\""+endBirthDay+"\"}}},");
                }
            }


            if(StringUtils.isNotEmpty(participantsDateS) && StringUtils.isNotEmpty(participantsDateE)){
                builder.append("{\"range\":{\"fadate\":{\"gte\":\""+participantsDateS+"\",\"lte\":\""+participantsDateE+"\"}}},");
            }else{

                if(StringUtils.isNotEmpty(participantsDateS)){
                    builder.append("{\"range\":{\"fadate\":{\"gte\":\""+participantsDateS+"\"}}},");
                }
                if(StringUtils.isNotEmpty(participantsDateE)){
                    builder.append("{\"range\":{\"fadate\":{\"lte\":\""+participantsDateE+"\"}}},");
                }
            }

            if(StringUtils.isNotEmpty(PIncomeS) && StringUtils.isNotEmpty(PIncomeE)){
                builder.append("{\"range\":{\"pincome\":{\"gte\":\""+PIncomeS+"\",\"lte\":\""+PIncomeE+"\"}}},");
            }else{

                if(StringUtils.isNotEmpty(PIncomeS)){
                    builder.append("{\"range\":{\"pincome\":{\"gte\":\""+PIncomeS+"\"}}},");
                }
                if(StringUtils.isNotEmpty(PIncomeE)){
                    builder.append("{\"range\":{\"pincome\":{\"lte\":\""+PIncomeE+"\"}}},");
                }
            }

            if(StringUtils.isNotEmpty(yx)){
                if (yx.equals("0")){
                    builder.append("{\"range\":{\"novpolicy\":{\"gt\":\""+yx+"\"}}},");
                }else
                {
                    builder.append("{\"prefix\":{\"label91\":\"失效客户\"}},");
                }
            }

            if(StringUtils.isNotEmpty(education)){
                String[] edus = education.split("&");
                builder.append("{\"terms\":{\"education\":[");
                for (String edu:edus) {
                    builder.append("\""+edu+"\",");
                }
                builder.delete(builder.length()-1,builder.length());
                builder.append ("]}},");
            }

            if(StringUtils.isNotEmpty(gzgw) && "0".equals(gzgw)){
                builder.append("{\"prefix\":{\"label73\":\"关注官微\"}},");
            }

            if(StringUtils.isNotEmpty(zcgw) && "0".equals(zcgw)){
                builder.append("{\"prefix\":{\"label74\":\"注册官微\"}},");
            }

            if(StringUtils.isNotEmpty(lp) && "0".equals(lp)){
                builder.append("{\"prefix\":{\"label57\":\"最近一次理赔类型\"}},");
            }

            if(StringUtils.isNotEmpty(gender) || StringUtils.isNotEmpty(NOVPolicy) || StringUtils.isNotEmpty(custtype)
                    || StringUtils.isNotEmpty(custclass) || StringUtils.isNotEmpty(sobirth) || StringUtils.isNotEmpty(SCustID)
                    || StringUtils.isNotEmpty(beginBirthDay) || StringUtils.isNotEmpty(endBirthDay) || StringUtils.isNotEmpty(education)
                    || StringUtils.isNotEmpty(PIncomeS) || StringUtils.isNotEmpty(PIncomeE) || (StringUtils.isNotEmpty(gzgw) && "0".equals(gzgw))
                    || (StringUtils.isNotEmpty(zcgw) && "0".equals(zcgw)) || StringUtils.isNotEmpty(yx)
                    || (StringUtils.isNotEmpty(lp) && "0".equals(lp)) || /*StringUtils.isNotEmpty(deptCode) ||*/
                    StringUtils.isNotEmpty(participantsDateS) || StringUtils.isNotEmpty(participantsDateE) ||
                    StringUtils.isNotEmpty(policybelong) || StringUtils.isNotEmpty(name) || StringUtils.isNotEmpty(mobile)
                    || StringUtils.isNotEmpty(fadate)){
                builder.delete(builder.length()-1,builder.length());
            }

            builder.append("],");//must结束
            builder.append("\"must_not\":[");//must_not
            if(StringUtils.isNotEmpty(gzgw) && "1".equals(gzgw)){
                builder.append("{\"prefix\":{\"label73\":\"关注官微\"}},");
            }

            if(StringUtils.isNotEmpty(zcgw) && "1".equals(zcgw)){
                builder.append("{\"prefix\":{\"label74\":\"注册官微\"}},");
            }

            if(StringUtils.isNotEmpty(lp) && "1".equals(lp)){
                builder.append("{\"prefix\":{\"label57\":\"最近一次理赔类型\"}},");
            }

            if(
                    (StringUtils.isNotEmpty(gzgw) && "1".equals(gzgw))
                            || (StringUtils.isNotEmpty(zcgw) && "1".equals(zcgw))
                            || (StringUtils.isNotEmpty(lp) && "1".equals(lp))
                    ){
                builder.delete(builder.length()-1,builder.length());
            }

            builder.append("],");
            builder.append("\"should\":[");

            //客户证件号码
            if(StringUtils.isNotEmpty(idcard)){
                builder.append("{\"wildcard\":{\"idcard\":\"*"+idcard+"*\"}},");
                builder.append("{\"wildcard\":{\"passport\":\"*"+idcard+"*\"}},");
                builder.append("{\"wildcard\":{\"dlicense\":\"*"+idcard+"*\"}},");
                builder.append("{\"wildcard\":{\"otheridnumber\":\"*"+idcard+"*\"}},");
                builder.delete(builder.length()-1,builder.length());
            }


            builder.append("]");//should
            builder.append("}");//boot结束
            builder.append("},");//query结束
            builder.append("\"from\":\"0\",");//第几页
            builder.append("\"size\":\"10000\",");//查询几个
            builder.append("\"sort\":[{\"scustid\":{\"order\":\"asc\"}}],");//排序
            builder.append("\"aggs\":{}");//
            builder.append("}");


            HttpEntity entity = new NStringEntity(builder.toString(), ContentType.APPLICATION_JSON);
            String url = "/shanglifeecif.individual/_search?&scroll=5m";
            Request scriptRequest = new Request("GET", url);
            scriptRequest.setEntity(entity);
            Response responseData = restClient.performRequest(scriptRequest);
            String result = EntityUtils.toString(responseData.getEntity());
            JSONObject jsonObject = JSON.parseObject(result);
            JSONObject hitsobject = (JSONObject) jsonObject.get("hits");
            JSONArray array = JSON.parseArray(hitsobject.get("hits").toString());
            List<Customer> customers = new ArrayList<Customer>();
            String scroll_id = jsonObject.getString("_scroll_id");
            entity = new NStringEntity("{\n" +
                    "  \"scroll_id\": \""+scroll_id+"\",\n" +
                    "  \"scroll\": \"5m\"\n" +
                    "}", ContentType.APPLICATION_JSON);
            while (array.size() != 0) {

                // 这里填写你的业务逻辑，即对每一条数据的处理

                for (int i = 0; i <array.size() ; i++) {
                    JSONObject json = (JSONObject)array.get(i);
                    Map mp = new HashMap();
                    Customer customer = new Customer();
                    json.getString("_source");
                    mp.put("id",json.getString("_id"));
                    JSONObject jsonOb = JSON.parseObject(json.getString("_source"));
                    mp.putAll(JSON.parseObject(json.getString("_source"),Map.class));
                    customer.setScustId("\t"+jsonOb.getString("scustid"));
                    customer.setCustId(jsonOb.getString("custid"));
                    customer.setName(jsonOb.getString("name"));
                    customer.setSex("0".equals(jsonOb.getString("gender"))?"男":"女");
                    customer.setCustType(jsonOb.getString("custtype"));
                    customer.setEducation(jsonOb.getString("education"));
                    customer.setVaild(StringUtils.isEmpty(jsonOb.getString("label91"))?"有效":"无效");
                    customer.setpIncome(jsonOb.getString("pincome"));
                    customer.setNum(jsonOb.getString("label25"));
                    String ral = jsonOb.getString("ral");
                    if(StringUtils.isNotBlank(ral)){
                        List<Map> list = AddressResolutionUtil.addressResolution(ral);
                        if(list!=null && list.size()>0){
                            customer.setProvince(list.get(0).get("province").toString());
                        }
                    }
                    if(mp.containsKey("birthday")){
                        String birthday = jsonOb.getString("birthday");
                        mp.put("birthday", birthday);
                        mp.put("age",DateUtils.getAge(DateUtils.dateTime(DateUtils.YYYY_MM_DD,birthday)));
                        customer.setBirthDay("\t"+birthday);
                        customer.setAge(DateUtils.getAge(DateUtils.dateTime(DateUtils.YYYY_MM_DD,birthday)));
                    }

                    String regex = "(\\w{4})(\\w+)(\\w{3})";
                    if(mp.containsKey("idcard")){
                        customer.setIdType("身份证");
                        customer.setIdCard("\t"+jsonOb.getString("idcard").replaceAll(regex, "$1****$3"));
                    }else if(mp.containsKey("passport")){
                        customer.setIdType("护照");
                        customer.setIdCard("\t"+jsonOb.getString("passport").replaceAll(regex, "$1****$3"));
                    }else if(mp.containsKey("dlicense")){
                        customer.setIdType("驾驶证");
                        customer.setIdCard("\t"+jsonOb.getString("dlicense").replaceAll(regex, "$1****$3"));
                    }else if(mp.containsKey("otheridnumber")){
                        customer.setIdType("其它证件类型");
                        customer.setIdCard("\t"+jsonOb.getString("otheridnumber").replaceAll(regex, "$1****$3"));
                    }

                    if(mp.containsKey("custclass")){
                        String custclazz = jsonOb.getString("custclass");
                        switch (custclazz){
                            case "1":
                                customer.setCustClass("钻石");
                                break;
                            case "2":
                                customer.setCustClass("白金");
                                break;
                            case "3":
                                customer.setCustClass("黄金");
                                break;
                            case "4":
                                customer.setCustClass("普通");
                                break;
                            default:
                                customer.setCustClass("无");
                        }

                    }else{
                        customer.setCustClass("无");
                    }

                    customers.add(customer);
                }

                // 继续向后查询
                scriptRequest = new Request("GET", "/_search/scroll");
                scriptRequest.setEntity(entity);
                responseData = restClient.performRequest(scriptRequest);
                result = EntityUtils.toString(responseData.getEntity());
                jsonObject = JSON.parseObject(result);
                hitsobject = (JSONObject) jsonObject.get("hits");
                array = JSON.parseArray(hitsobject.get("hits").toString());
            }
            Map<String, String> header = new LinkedHashMap<>();
            header.put("scustId", "核心客户号");
            header.put("custId", "客户号");
            header.put("name", "姓名");
            header.put("idType", "证件类型");
            header.put("idCard", "证件号码");
            header.put("sex", "性别");
            header.put("age", "年龄");
            header.put("birthDay", "出生日期");
            header.put("custClass", "客户等级");
            header.put("custType", "客户类型");
            header.put("province", "省份");
            header.put("education", "学历");
            header.put("vaild", "是否有效客户");
            /*header.put("policyBelong", "业绩归属");*/
            header.put("pIncome", "年收入(万)");
            header.put("num", "保单件数");
            CsvUtil.export(request, response, "customer", header, customers);
            logger.info("导出出时间："+DateUtils.getTime());
        }catch (Exception e){
            logger.info("导出出错："+DateUtils.getTime());
            e.printStackTrace();
        }

    }

    /**
     * 导出四要素疑似相同指姓名、性别、证件类型、手机号一致
     * @param response
     * @param request
     */
    @GetMapping("/exportSameZero")
    public void exportSameZero(HttpServletResponse response, HttpServletRequest request)
    {
        logger.info("导出四要素进时间："+DateUtils.getTime());

        try{

            Request scriptRequest = new Request("GET", "/shanglifeecif.samecustomer/_search?sort=name:asc&q=sametype:0&size=10000&scroll=5m");
            Response responseData = restClient.performRequest(scriptRequest);
            String result = EntityUtils.toString(responseData.getEntity());
            JSONObject jsonObject = JSON.parseObject(result);
            JSONObject hitsobject = (JSONObject) jsonObject.get("hits");
            JSONArray array = JSON.parseArray(hitsobject.get("hits").toString());
            List<Customer> customers = new ArrayList<Customer>();
            String scroll_id = jsonObject.getString("_scroll_id");
            HttpEntity entity = new NStringEntity("{\n" +
                    "  \"scroll_id\": \""+scroll_id+"\",\n" +
                    "  \"scroll\": \"5m\"\n" +
                    "}", ContentType.APPLICATION_JSON);
            while (array.size() != 0) {

                // 这里填写你的业务逻辑，即对每一条数据的处理

                for (int i = 0; i <array.size() ; i++) {
                    JSONObject json = (JSONObject)array.get(i);
                    Map mp = new HashMap();
                    Customer customer = new Customer();
                    json.getString("_source");
                    mp.put("id",json.getString("_id"));
                    JSONObject jsonOb = JSON.parseObject(json.getString("_source"));
                    mp.putAll(JSON.parseObject(json.getString("_source"),Map.class));
                    customer.setScustId("\t"+jsonOb.getString("scustid"));
                    customer.setName(jsonOb.getString("name"));
                    customer.setSex("0".equals(jsonOb.getString("gender"))?"男":"女");
                    if(mp.containsKey("birthday")){
                        String birthday = jsonOb.getString("birthday");
                        mp.put("birthday", birthday);
                        customer.setBirthDay("\t"+birthday);
                        customer.setAge(DateUtils.getAge(DateUtils.dateTime(DateUtils.YYYY_MM_DD,jsonOb.getString("birthday"))));
                    }

                    if(mp.containsKey("idcard")){
                        customer.setIdType("身份证");
                        customer.setIdCard("\t"+jsonOb.getString("idcard"));
                    }else if(mp.containsKey("passport")){
                        customer.setIdType("护照");
                        customer.setIdCard("\t"+jsonOb.getString("passport"));
                    }else if(mp.containsKey("dlicense")){
                        customer.setIdType("驾驶证");
                        customer.setIdCard("\t"+jsonOb.getString("dlicense"));
                    }else if(mp.containsKey("otheridnumber")){
                        customer.setIdType("其它证件类型");
                        customer.setIdCard("\t"+jsonOb.getString("otheridnumber"));
                    }

                    customers.add(customer);
                }

                // 继续向后查询
                scriptRequest = new Request("GET", "/_search/scroll");
                scriptRequest.setEntity(entity);
                responseData = restClient.performRequest(scriptRequest);
                result = EntityUtils.toString(responseData.getEntity());
                jsonObject = JSON.parseObject(result);
                hitsobject = (JSONObject) jsonObject.get("hits");
                array = JSON.parseArray(hitsobject.get("hits").toString());
            }
            Map<String, String> header = new LinkedHashMap<>();
            header.put("scustId", "核心客户号");
            header.put("name", "姓名");
            header.put("idType", "证件类型");
            header.put("idCard", "证件号码");
            header.put("sex", "性别");
            header.put("birthDay", "出生日期");
            CsvUtil.export(request, response, "customer", header, customers);
            logger.info("导出四要素出时间："+DateUtils.getTime());
        }catch (Exception e){
            logger.info("导出四要素出错："+DateUtils.getTime());
            e.printStackTrace();
        }

    }

    /**
     * 导出三要素疑似相同指姓名、性别、出生日期一致
     * @param response
     * @param request
     */
    @GetMapping("/exportSameOne")
    public void exportSameOne(HttpServletResponse response, HttpServletRequest request)
    {
        logger.info("导出三要素进时间："+DateUtils.getTime());

        try{

            Request scriptRequest = new Request("GET", "/shanglifeecif.samecustomer/_search?sort=name:asc&q=sametype:1&size=10000&scroll=5m");
            Response responseData = restClient.performRequest(scriptRequest);
            String result = EntityUtils.toString(responseData.getEntity());
            JSONObject jsonObject = JSON.parseObject(result);
            JSONObject hitsobject = (JSONObject) jsonObject.get("hits");
            JSONArray array = JSON.parseArray(hitsobject.get("hits").toString());
            List<Customer> customers = new ArrayList<Customer>();
            String scroll_id = jsonObject.getString("_scroll_id");
            HttpEntity entity = new NStringEntity("{\n" +
                    "  \"scroll_id\": \""+scroll_id+"\",\n" +
                    "  \"scroll\": \"5m\"\n" +
                    "}", ContentType.APPLICATION_JSON);
            while (array.size() != 0) {

                // 这里填写你的业务逻辑，即对每一条数据的处理

                for (int i = 0; i <array.size() ; i++) {
                    JSONObject json = (JSONObject)array.get(i);
                    Map mp = new HashMap();
                    Customer customer = new Customer();
                    json.getString("_source");
                    mp.put("id",json.getString("_id"));
                    JSONObject jsonOb = JSON.parseObject(json.getString("_source"));
                    mp.putAll(JSON.parseObject(json.getString("_source"),Map.class));
                    customer.setScustId("\t"+jsonOb.getString("scustid"));
                    customer.setName(jsonOb.getString("name"));
                    customer.setSex("0".equals(jsonOb.getString("gender"))?"男":"女");
                    if(mp.containsKey("birthday")){
                        String birthday = jsonOb.getString("birthday");
                        mp.put("birthday", birthday);
                        customer.setBirthDay(birthday);
                        customer.setAge(DateUtils.getAge(DateUtils.dateTime(DateUtils.YYYY_MM_DD,jsonOb.getString("birthday"))));
                    }

                    if(mp.containsKey("idcard")){
                        customer.setIdType("身份证");
                        customer.setIdCard("\t"+jsonOb.getString("idcard"));
                    }else if(mp.containsKey("passport")){
                        customer.setIdType("护照");
                        customer.setIdCard("\t"+jsonOb.getString("passport"));
                    }else if(mp.containsKey("dlicense")){
                        customer.setIdType("驾驶证");
                        customer.setIdCard("\t"+jsonOb.getString("dlicense"));
                    }else if(mp.containsKey("otheridnumber")){
                        customer.setIdType("其它证件类型");
                        customer.setIdCard("\t"+jsonOb.getString("otheridnumber"));
                    }

                    customers.add(customer);
                }

                // 继续向后查询
                scriptRequest = new Request("GET", "/_search/scroll");
                scriptRequest.setEntity(entity);
                responseData = restClient.performRequest(scriptRequest);
                result = EntityUtils.toString(responseData.getEntity());
                jsonObject = JSON.parseObject(result);
                hitsobject = (JSONObject) jsonObject.get("hits");
                array = JSON.parseArray(hitsobject.get("hits").toString());
            }
            Map<String, String> header = new LinkedHashMap<>();
            header.put("scustId", "核心客户号");
            header.put("name", "姓名");
            header.put("idType", "证件类型");
            header.put("idCard", "证件号码");
            header.put("sex", "性别");
            header.put("birthDay", "出生日期");
            CsvUtil.export(request, response, "customer", header, customers);
            logger.info("导出三要素出时间："+DateUtils.getTime());
        }catch (Exception e){
            logger.info("导出三要素出错："+DateUtils.getTime());
            e.printStackTrace();
        }

    }

    /**
     * 导出两要素疑似相同指姓名、性别、出生日期一致
     * @param response
     * @param request
     */
    @GetMapping("/exportSameTwo")
    public void exportSameTwo(HttpServletResponse response, HttpServletRequest request)
    {
        logger.info("导出两要素进时间："+DateUtils.getTime());

        try{

            Request scriptRequest = new Request("GET", "/shanglifeecif.samecustomer/_search?sort=name:asc&q=sametype:2&size=10000&scroll=5m");
            Response responseData = restClient.performRequest(scriptRequest);
            String result = EntityUtils.toString(responseData.getEntity());
            JSONObject jsonObject = JSON.parseObject(result);
            JSONObject hitsobject = (JSONObject) jsonObject.get("hits");
            JSONArray array = JSON.parseArray(hitsobject.get("hits").toString());
            List<Customer> customers = new ArrayList<Customer>();
            String scroll_id = jsonObject.getString("_scroll_id");
            HttpEntity entity = new NStringEntity("{\n" +
                    "  \"scroll_id\": \""+scroll_id+"\",\n" +
                    "  \"scroll\": \"5m\"\n" +
                    "}", ContentType.APPLICATION_JSON);
            while (array.size() != 0) {

                // 这里填写你的业务逻辑，即对每一条数据的处理

                for (int i = 0; i <array.size() ; i++) {
                    JSONObject json = (JSONObject)array.get(i);
                    Map mp = new HashMap();
                    Customer customer = new Customer();
                    json.getString("_source");
                    mp.put("id",json.getString("_id"));
                    JSONObject jsonOb = JSON.parseObject(json.getString("_source"));
                    mp.putAll(JSON.parseObject(json.getString("_source"),Map.class));
                    customer.setScustId("\t"+jsonOb.getString("scustid"));
                    customer.setName(jsonOb.getString("name"));
                    customer.setSex("0".equals(jsonOb.getString("gender"))?"男":"女");
                    if(mp.containsKey("birthday")){
                        String birthday = jsonOb.getString("birthday");
                        mp.put("birthday", birthday);
                        customer.setBirthDay("\t"+birthday);
                        customer.setAge(DateUtils.getAge(DateUtils.dateTime(DateUtils.YYYY_MM_DD,jsonOb.getString("birthday"))));
                    }

                    if(mp.containsKey("idcard")){
                        customer.setIdType("身份证");
                        customer.setIdCard("\t"+jsonOb.getString("idcard"));
                    }else if(mp.containsKey("passport")){
                        customer.setIdType("护照");
                        customer.setIdCard("\t"+jsonOb.getString("passport"));
                    }else if(mp.containsKey("dlicense")){
                        customer.setIdType("驾驶证");
                        customer.setIdCard("\t"+jsonOb.getString("dlicense"));
                    }else if(mp.containsKey("otheridnumber")){
                        customer.setIdType("其它证件类型");
                        customer.setIdCard("\t"+jsonOb.getString("otheridnumber"));
                    }

                    customers.add(customer);
                }

                // 继续向后查询
                scriptRequest = new Request("GET", "/_search/scroll");
                scriptRequest.setEntity(entity);
                responseData = restClient.performRequest(scriptRequest);
                result = EntityUtils.toString(responseData.getEntity());
                jsonObject = JSON.parseObject(result);
                hitsobject = (JSONObject) jsonObject.get("hits");
                array = JSON.parseArray(hitsobject.get("hits").toString());
            }
            Map<String, String> header = new LinkedHashMap<>();
            header.put("scustId", "核心客户号");
            header.put("name", "姓名");
            header.put("idType", "证件类型");
            header.put("idCard", "证件号码");
            header.put("sex", "性别");
            header.put("birthDay", "出生日期");
            CsvUtil.export(request, response, "customer", header, customers);

            logger.info("导出两要素出时间："+DateUtils.getTime());
        }catch (Exception e){
            logger.info("导出两要素出错："+DateUtils.getTime());
            e.printStackTrace();
        }

    }

    /**
     * 导出数据清洗数据
     * @param response
     * @param request
     */
    @GetMapping("/dataCleaning")
    public void dataCleaning(HttpServletResponse response, HttpServletRequest request,String sameType)
    {
        logger.info("导出数据清洗进时间："+DateUtils.getTime());

        try{

            Request scriptRequest = new Request("GET", "/shanglifeecif.samecustomer/_search?sort=name:asc&q=sametype:"+sameType+"&size=10000&scroll=5m");
            Response responseData = restClient.performRequest(scriptRequest);
            String result = EntityUtils.toString(responseData.getEntity());
            JSONObject jsonObject = JSON.parseObject(result);
            JSONObject hitsobject = (JSONObject) jsonObject.get("hits");
            JSONArray array = JSON.parseArray(hitsobject.get("hits").toString());
            List<Customer> customers = new ArrayList<Customer>();
            String scroll_id = jsonObject.getString("_scroll_id");
            HttpEntity entity = new NStringEntity("{\n" +
                    "  \"scroll_id\": \""+scroll_id+"\",\n" +
                    "  \"scroll\": \"5m\"\n" +
                    "}", ContentType.APPLICATION_JSON);
            while (array.size() != 0) {

                // 这里填写你的业务逻辑，即对每一条数据的处理

                for (int i = 0; i <array.size() ; i++) {
                    JSONObject json = (JSONObject)array.get(i);
                    Map mp = new HashMap();
                    Customer customer = new Customer();
                    json.getString("_source");
                    mp.put("id",json.getString("_id"));
                    JSONObject jsonOb = JSON.parseObject(json.getString("_source"));
                    mp.putAll(JSON.parseObject(json.getString("_source"),Map.class));
                    customer.setScustId("\t"+jsonOb.getString("scustid"));
                    customer.setName(jsonOb.getString("name"));
                    customer.setSex("0".equals(jsonOb.getString("gender"))?"男":"女");
                    if(mp.containsKey("birthday")){
                        String birthday = jsonOb.getString("birthday");
                        mp.put("birthday", birthday);
                        customer.setBirthDay("\t"+birthday);
                        customer.setAge(DateUtils.getAge(DateUtils.dateTime(DateUtils.YYYY_MM_DD,jsonOb.getString("birthday"))));
                    }

                    if(mp.containsKey("idcard")){
                        customer.setIdType("身份证");
                        customer.setIdCard("\t"+jsonOb.getString("idcard"));
                    }else if(mp.containsKey("passport")){
                        customer.setIdType("护照");
                        customer.setIdCard("\t"+jsonOb.getString("passport"));
                    }else if(mp.containsKey("dlicense")){
                        customer.setIdType("驾驶证");
                        customer.setIdCard("\t"+jsonOb.getString("dlicense"));
                    }else if(mp.containsKey("otheridnumber")){
                        customer.setIdType("其它证件类型");
                        customer.setIdCard("\t"+jsonOb.getString("otheridnumber"));
                    }

                    customers.add(customer);
                }

                // 继续向后查询
                scriptRequest = new Request("GET", "/_search/scroll");
                scriptRequest.setEntity(entity);
                responseData = restClient.performRequest(scriptRequest);
                result = EntityUtils.toString(responseData.getEntity());
                jsonObject = JSON.parseObject(result);
                hitsobject = (JSONObject) jsonObject.get("hits");
                array = JSON.parseArray(hitsobject.get("hits").toString());
            }
            Map<String, String> header = new LinkedHashMap<>();
            header.put("scustId", "核心客户号");
            header.put("name", "姓名");
            header.put("idType", "证件类型");
            header.put("idCard", "证件号码");
            header.put("sex", "性别");
            header.put("birthDay", "出生日期");
            CsvUtil.export(request, response, "customer", header, customers);

            logger.info("导出数据清洗出时间："+DateUtils.getTime());
        }catch (Exception e){
            logger.info("导出数据清洗出错："+DateUtils.getTime());
            e.printStackTrace();
        }

    }

}
